import pygame
import math

# Initialize pygame
pygame.init()

# Screen dimensions
WIDTH, HEIGHT = 1920, 1080
screen = pygame.display.set_mode((WIDTH, HEIGHT))
pygame.display.set_caption("AI Car on a Complex Track")

# Colors
WHITE = (255, 255, 255)
GRAY = (50, 50, 50)
BLUE = (0, 0, 255)
GREEN = (0, 255, 0)
RED = (255, 0, 0)

# Clock
clock = pygame.time.Clock()

# Car properties
class Car:
    def __init__(self, x, y, color):
        self.x = x
        self.y = y
        self.angle = 0  # Initial angle
        self.speed = 0
        self.max_speed = 6
        self.acceleration = 0.2
        self.friction = 0.05
        self.steering = 5
        self.image = pygame.Surface((80, 40), pygame.SRCALPHA)
        self.color = color
        self.draw_car()

    def draw_car(self):
        pygame.draw.rect(self.image, self.color, (0, 0, 80, 40))

    def move(self, target_x, target_y):
        # Calculate angle to the target
        dx = target_x - self.x
        dy = target_y - self.y
        target_angle = math.degrees(math.atan2(-dy, dx))

        # Adjust angle toward the target with smoothing
        angle_diff = (target_angle - self.angle + 360) % 360
        if angle_diff > 180:
            angle_diff -= 360
        # Allow gradual steering
        self.angle += max(min(angle_diff, self.steering), -self.steering)

        # Accelerate the car
        self.speed = min(self.speed + self.acceleration, self.max_speed)

        # Apply movement
        rad_angle = math.radians(-self.angle)
        self.x += self.speed * math.cos(rad_angle)
        self.y += self.speed * math.sin(rad_angle)

        # Apply friction to slow down when not accelerating
        if self.speed > 0:
            self.speed -= self.friction
        elif self.speed < 0:
            self.speed += self.friction

    def draw(self, screen):
        rotated_image = pygame.transform.rotate(self.image, self.angle)
        rect = rotated_image.get_rect(center=(self.x, self.y))
        screen.blit(rotated_image, rect.topleft)

# Define waypoints for the track (complex track)
waypoints = [
    (200, 300), (400, 200), (600, 300), (800, 500), (1000, 700),
    (1200, 800), (1400, 700), (1600, 500), (1400, 300), (1200, 200),
    (1000, 100), (800, 200), (600, 100), (400, 300), (200, 500),
    (200, 300)  # Finish/Start Line
]

# Create an AI car
ai_car = Car(200, 300, BLUE)
current_waypoint = 0

# Game loop
running = True
while running:
    screen.fill(GRAY)

    # Draw the track (connect waypoints with lines)
    for i in range(len(waypoints) - 1):
        pygame.draw.line(screen, GREEN, waypoints[i], waypoints[i + 1], 5)

    # Draw waypoints
    for waypoint in waypoints:
        pygame.draw.circle(screen, RED, waypoint, 10)

    # Event handling
    for event in pygame.event.get():
        if event.type == pygame.QUIT:
            running = False

    # AI logic: Move the car toward the current waypoint
    target_x, target_y = waypoints[current_waypoint]
    ai_car.move(target_x, target_y)

    # Check if the car reached the waypoint
    if math.hypot(ai_car.x - target_x, ai_car.y - target_y) < 20:
        current_waypoint = (current_waypoint + 1) % len(waypoints)  # Move to the next waypoint

    # Draw the car
    ai_car.draw(screen)

    # Update display
    pygame.display.flip()
    clock.tick(60)

# Quit pygame
pygame.quit()
