import pygame
import random
import time

# Initialize pygame
pygame.init()

# Constants
WIDTH, HEIGHT = 800, 600
SNAKE_BLOCK = 20
GRID_WIDTH, GRID_HEIGHT = WIDTH // SNAKE_BLOCK, HEIGHT // SNAKE_BLOCK
WHITE = (255, 255, 255)
RED = (213, 50, 80)
GREEN = (0, 255, 0)
YELLOW = (255, 255, 102)
BLACK = (0, 0, 0)

# Initialize screen
screen = pygame.display.set_mode((WIDTH, HEIGHT))
pygame.display.set_caption("Snake Game with Hamiltonian Cycle")
clock = pygame.time.Clock()

# Font
font_style = pygame.font.SysFont("bahnschrift", 25)

def message(msg, color, x, y):
    mesg = font_style.render(msg, True, color)
    screen.blit(mesg, [x, y])

# Generate the Hamiltonian Cycle
def generate_hamiltonian_cycle(width, height):
    cycle = []
    for y in range(height):
        if y % 2 == 0:  # Left-to-right on even rows
            for x in range(width):
                cycle.append((x, y))
        else:  # Right-to-left on odd rows
            for x in range(width - 1, -1, -1):
                cycle.append((x, y))
    return cycle

# Initialize Hamiltonian cycle
hamiltonian_cycle = generate_hamiltonian_cycle(GRID_WIDTH, GRID_HEIGHT)

# Game loop
def game_loop():
    snake = [(GRID_WIDTH // 2, GRID_HEIGHT // 2)]
    snake_length = 1
    direction_index = 0  # Current position in the Hamiltonian cycle
    food = (GRID_WIDTH // 4, GRID_HEIGHT // 4)  # Initial food position
    snake_speed = 15

    def place_food():
        nonlocal food
        while True:
            fx = random.randint(0, GRID_WIDTH - 1)
            fy = random.randint(0, GRID_HEIGHT - 1)
            if (fx, fy) not in snake:
                food = (fx, fy)
                break

    def draw_snake():
        for block in snake:
            pygame.draw.rect(screen, GREEN, [block[0] * SNAKE_BLOCK, block[1] * SNAKE_BLOCK, SNAKE_BLOCK, SNAKE_BLOCK])

    def draw_food():
        pygame.draw.rect(screen, YELLOW, [food[0] * SNAKE_BLOCK, food[1] * SNAKE_BLOCK, SNAKE_BLOCK, SNAKE_BLOCK])

    game_over = False

    while not game_over:
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                pygame.quit()
                quit()
            if event.type == pygame.KEYDOWN:
                if event.key == pygame.K_1:
                    snake_speed = 15
                elif event.key == pygame.K_2:
                    snake_speed = 30
                elif event.key == pygame.K_3:
                    snake_speed = 3000

        # Move snake along Hamiltonian cycle
        direction_index = (direction_index + 1) % len(hamiltonian_cycle)
        next_position = hamiltonian_cycle[direction_index]

        # Check for collision with itself
        if next_position in snake[:-1]:
            game_over = True
            reason = "You ran into yourself!"
            break

        # Add new position to the snake
        snake.append(next_position)

        # Check if the snake eats food
        if next_position == food:
            snake_length += 10
            place_food()
        
        # Trim the snake to its current length
        if len(snake) > snake_length:
            snake.pop(0)

        # Render game
        screen.fill(BLACK)
        draw_snake()
        draw_food()
        pygame.display.update()

        clock.tick(snake_speed)

    # Game over screen
    screen.fill(BLACK)
    message(f"Game Over! {reason}", RED, WIDTH // 6, HEIGHT // 3)
    pygame.display.update()
    time.sleep(2)
    pygame.quit()
    quit()

game_loop()
